﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Globalization;
using System.Threading.Tasks;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace Post9696
{
    /// <summary>
    /// This sample shows how to add features to layer; 
    /// then build index,
    /// then pre-cache from 01-05, you can use the static method to pre-build your own cache.
    /// </summary>
    public partial class MainWindow : Window
    {
        private static readonly int maxRecordsToDraw = 1000000;
        private static readonly RectangleShape worldExtent = new RectangleShape(-180, 90, 180, -90);
        private static readonly string cachePath = "e:\\RandomPointCache\\";
        private static readonly string cacheID = "cacheID";

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Map1_Loaded(object sender, RoutedEventArgs e)
        {
            Random random = new Random();
            InMemoryFeatureLayer featureLayer = new InMemoryFeatureLayer();
            featureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City2;
            featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            Task task = new Task((parameter) =>
            {
                featureLayer.Open();
                featureLayer.EditTools.BeginTransaction();
                for (int i = 0; i < maxRecordsToDraw; i++)
                {
                    PointShape pointShape = new PointShape(random.NextDouble() * 360d - 180d, random.NextDouble() * 180d - 90d);
                    featureLayer.EditTools.Add(pointShape);
                    if (i % 100000 == 0)
                    {
                        Dispatcher.BeginInvoke(new Action(() =>
                        {
                            ProgressBar1.Value = i * 100 / maxRecordsToDraw;
                            TextBlock1.Text = String.Format(CultureInfo.InvariantCulture, "{0}% features inserted.", ProgressBar1.Value);
                        }));
                    }
                }
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    ProgressBar1.IsIndeterminate = true;
                    TextBlock1.Text = "Building Index...";
                }));
                featureLayer.EditTools.CommitTransaction();

                Dispatcher.BeginInvoke(new Action(() =>
                {
                    TextBlock1.Text = "Pre-cache for zoom level 01";
                }));
                GenerateCacheFiles(new Collection<Layer>() { featureLayer }, featureLayer.ZoomLevelSet.ZoomLevel01.Scale, worldExtent, GeographyUnit.DecimalDegree, cachePath, cacheID);
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    TextBlock1.Text = "Pre-cache for zoom level 02";
                }));
                GenerateCacheFiles(new Collection<Layer>() { featureLayer }, featureLayer.ZoomLevelSet.ZoomLevel02.Scale, worldExtent, GeographyUnit.DecimalDegree, cachePath, cacheID);
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    TextBlock1.Text = "Pre-cache for zoom level 03";
                }));
                GenerateCacheFiles(new Collection<Layer>() { featureLayer }, featureLayer.ZoomLevelSet.ZoomLevel03.Scale, worldExtent, GeographyUnit.DecimalDegree, cachePath, cacheID);
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    TextBlock1.Text = "Pre-cache for zoom level 04";
                }));
                GenerateCacheFiles(new Collection<Layer>() { featureLayer }, featureLayer.ZoomLevelSet.ZoomLevel04.Scale, worldExtent, GeographyUnit.DecimalDegree, cachePath, cacheID);
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    TextBlock1.Text = "Pre-cache for zoom level 05";
                }));
                GenerateCacheFiles(new Collection<Layer>() { featureLayer }, featureLayer.ZoomLevelSet.ZoomLevel05.Scale, worldExtent, GeographyUnit.DecimalDegree, cachePath, cacheID);
            }, featureLayer);

            task.ContinueWith((parameter) =>
            {
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    ProgressBar1.Visibility = Visibility.Collapsed;
                    TextBlock1.Visibility = Visibility.Collapsed;

                    Map1.MapUnit = GeographyUnit.DecimalDegree;
                    LayerOverlay layerOverlay = new LayerOverlay();
                    layerOverlay.TileCache = new FileBitmapTileCache(cachePath, cacheID);
                    layerOverlay.Layers.Add(featureLayer);
                    Map1.Overlays.Add(layerOverlay);
                    Map1.Refresh();
                }));
            });

            task.Start();
        }

        private static void GenerateCacheFiles(IEnumerable<Layer> layers, double scale, RectangleShape restrictedExtent, GeographyUnit mapUnit, string cacheDirectory, string cacheId)
        {
            // create a MapSuite default tile matrix object.
            MapSuiteTileMatrix matrix = new MapSuiteTileMatrix(scale, 256, 256, mapUnit);

            // get cells in the specified extent.
            Collection<TileMatrixCell> cells = matrix.GetIntersectingCells(restrictedExtent);

            // create a tile cache object with the cache directory and cache id.
            FileBitmapTileCache tileCache = new FileBitmapTileCache(cacheDirectory, cacheId);

            foreach (TileMatrixCell cell in cells)
            {
                // create a bitmap whoes size equals to the tile size.
                using (Bitmap bitmap = new Bitmap(256, 256))
                {
                    // create a GeoCanvas which for drawing the passed layers.
                    GdiPlusGeoCanvas geoCanvas = new GdiPlusGeoCanvas();
                    geoCanvas.BeginDrawing(bitmap, cell.BoundingBox, mapUnit);
                    foreach (Layer layer in layers)
                    {
                        lock (layer)
                        {
                            if (!layer.IsOpen) { layer.Open(); }
                            layer.Draw(geoCanvas, new Collection<SimpleCandidate>());
                        }
                    }

                    // you can commend this line out because it's just for mark a cache label in order to see if it works with our samples.
                    geoCanvas.DrawText("Cache", new GeoFont("Arial", 12), new GeoSolidBrush(GeoColor.SimpleColors.Black), new Collection<ScreenPointF>() { new ScreenPointF(128f, 128f) }, DrawingLevel.LabelLevel);
                    geoCanvas.EndDrawing();

                    // create tile object to maintain current tile information.
                    BitmapTile tile = new BitmapTile(bitmap, cell.BoundingBox, scale);

                    // save tile.
                    tileCache.SaveTile(tile);
                }
            }
        }
    }
}
